# Generated by Django 2.0.9 on 2018-11-19 12:05

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [("cast", "0010_blog_email")]

    operations = [
        migrations.AddField(
            model_name="post",
            name="uuid",
            field=models.UUIDField(default=uuid.uuid4, editable=False),
        )
    ]



# Generated by Django 2.0.9 on 2018-11-19 16:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("cast", "0014_auto_20181119_1616")]

    operations = [
        migrations.AddField(
            model_name="blog",
            name="keywords",
            field=models.CharField(
                blank=True,
                help_text="A comma-demlimitedlist of up to 12 words for iTunes\n            searches. Perhaps include misspellings of the title.",
                max_length=255,
                verbose_name="keywords",
            ),
        )
    ]



from django.core.management.base import BaseCommand
from django.core.files.storage import default_storage
from django.core.files.storage import get_storage_class

from ...models import File
from ...models import Image
from ...models import Video

from ...utils import storage_walk_paths


class Command(BaseCommand):
    help = (
        "show media files which are in the filesystem (s3, locale), "
        "but not in database and optionally delete them"
    )

    def add_arguments(self, parser):
        parser.add_argument(
            "--delete",
            action="store_true",
            dest="delete",
            default=False,
            help="Delete stale files instead of only showing them",
        )

    def get_paths(self, storage):
        paths = {}
        for path in storage_walk_paths(storage):
            size = storage.size(path)
            paths[path] = size
            # print(path, size / 2 ** 20)
        return paths

    def get_models_paths(self):
        paths = set()
        for image in Image.objects.all():
            for path in image.get_all_paths():
                paths.add(path)
        for video in Video.objects.all():
            for path in video.get_all_paths():
                paths.add(path)
        for misc_file in File.objects.all():
            for path in misc_file.get_all_paths():
                paths.add(path)
        return paths

    def handle(self, *args, **options):
        paths_from_models = self.get_models_paths()

        print("stale s3")
        s3 = get_storage_class("storages.backends.s3boto3.S3Boto3Storage")()
        s3_paths = self.get_paths(s3)
        stale_s3 = {}
        for path, size in s3_paths.items():
            if path not in paths_from_models:
                print(path)
                stale_s3[path] = size
        print("stale s3 size: {} Mb".format(sum(stale_s3.values()) / 2 ** 20))

        print("stale locale")
        locale_paths = self.get_paths(default_storage)
        stale_locale = {}
        for path, size in locale_paths.items():
            if path not in paths_from_models:
                print(path)
                stale_locale[path] = size
        print("stale locale size: {} Mb".format(sum(stale_locale.values()) / 2 ** 20))

        if options["delete"]:
            # for path in stale_s3.keys():
            #     s3.delete(path)
            for path in stale_locale.keys():
                default_storage.delete(path)



# Generated by Django 2.0.9 on 2018-11-19 17:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("cast", "0016_post_keywords")]

    operations = [
        migrations.AddField(
            model_name="blog",
            name="explicit",
            field=models.PositiveSmallIntegerField(
                choices=[(1, "yes"), (2, "no"), (3, "clean")],
                default=1,
                help_text="``Clean`` will put the clean iTunes graphic by it.",
                verbose_name="explicit",
            ),
        ),
        migrations.AddField(
            model_name="post",
            name="explicit",
            field=models.PositiveSmallIntegerField(
                choices=[(1, "yes"), (2, "no"), (3, "clean")],
                default=1,
                help_text="``Clean`` will put the clean iTunes graphic by it.",
                verbose_name="explicit",
            ),
        ),
    ]



from __future__ import absolute_import

from django import forms
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _

from .models import Post, Image, Video, ChapterMark


class MyDateTimeInput(forms.DateTimeInput):
    def render(self, *args, **kwargs):
        value = kwargs.get("value")
        if value is not None and not isinstance(value, str):
            kwargs["value"] = str(value.date())
        return super().render(*args, **kwargs)


class ChapterMarkForm(forms.ModelForm):
    class Meta:
        model = ChapterMark
        fields = ("audio", "start", "title", "link", "image")


class PostForm(forms.ModelForm):
    is_published = forms.BooleanField(required=False)
    pub_date = forms.DateTimeField(input_formats=["%Y-%m-%dT%H:%M"])

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["slug"].required = False
        self.fields["title"].widget.attrs["size"] = 80
        self.fields["pub_date"].required = False
        self.fields["pub_date"].widget = forms.DateTimeInput(
            attrs={"type": "datetime-local"}
        )
        self.fields["pub_date"].label = _("Publication date")
        self.fields["pub_date"].help_text = _(
            "Article will be published after this date/time."
        )

        self.fields["visible_date"].required = False
        self.fields["visible_date"].widget = MyDateTimeInput(attrs={"type": "date"})
        self.fields["visible_date"].label = _("Visible date")
        self.fields["visible_date"].help_text = _("Date to be shown above article.")

        self.fields["podcast_audio"].help_text = _(
            "The audio object to be used as podcast episode."
        )

        if self.instance.podcast_audio:
            self.fields["chaptermarks"] = forms.CharField(widget=forms.Textarea)

    def _set_pub_date(self, cleaned_data):
        pub_date = cleaned_data.get("pub_date")
        is_published = cleaned_data.get("is_published")
        if pub_date is None and is_published:
            cleaned_data["pub_date"] = timezone.now()
        return cleaned_data

    def _set_visible_date(self, cleaned_data):
        # dunno why this is necessary. 2019-02-26 jochen
        # visible_date is neither None in tests nor in notebook, but using
        # browser it raises null constraint error :/ wtf?
        visible_date = cleaned_data.get("visible_date")
        if visible_date is None:
            cleaned_data["visible_date"] = timezone.now()
        return cleaned_data

    def _clean_chaptermarks(self, cleaned_data):
        audio = self.instance.podcast_audio
        if audio:
            errors = []
            lines = cleaned_data.get("chaptermarks", "").split("\n")
            if len(lines) > 0:
                audio.chaptermarks.all().delete()
            for line in cleaned_data.get("chaptermarks", "").split("\n"):
                start, *parts = line.split()
                title = " ".join(parts)
                row = {
                    "audio": audio.pk,
                    "start": start,
                    "title": title,
                    # "link": None,
                    # "image": None
                }
                form = ChapterMarkForm(row)
                if form.is_valid():
                    form.save()
                else:
                    errors.append(form.errors)
                # TODO:
                # * image/link handling + tests
            if len(errors) > 0:
                self.add_error("chaptermarks", errors)
        return cleaned_data

    def clean(self):
        cleaned_data = super().clean()
        cleaned_data = self._set_pub_date(cleaned_data)
        cleaned_data = self._set_visible_date(cleaned_data)
        cleaned_data = self._clean_chaptermarks(cleaned_data)
        return cleaned_data

    class Meta:
        model = Post
        fields = [
            "title",
            "content",
            "pub_date",
            "visible_date",
            "is_published",
            "podcast_audio",
            "keywords",
            "explicit",
            "block",
            "slug",
        ]


class ImageForm(forms.ModelForm):
    class Meta:
        model = Image
        fields = ["original"]


class VideoForm(forms.ModelForm):
    class Meta:
        model = Video
        fields = ["original"]



# Generated by Django 2.1.3 on 2018-11-21 14:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("cast", "0019_auto_20181120_1024")]

    operations = [
        migrations.AddField(
            model_name="blog",
            name="itunes_categories",
            field=models.CharField(
                blank=True,
                default="",
                help_text="A json dict of itunes categories pointing to lists of subcategories. Taken from this list https://validator.w3.org/feed/docs/error/InvalidItunesCategory.html",
                max_length=512,
                verbose_name="itunes_categories",
            ),
        ),
        migrations.AlterField(
            model_name="blog",
            name="keywords",
            field=models.CharField(
                blank=True,
                default="",
                help_text="A comma-delimitedlist of up to 12 words for iTunes\n            searches. Perhaps include misspellings of the title.",
                max_length=255,
                verbose_name="keywords",
            ),
        ),
        migrations.AlterField(
            model_name="post",
            name="keywords",
            field=models.CharField(
                blank=True,
                default="",
                help_text="A comma-demlimitedlist of up to 12 words for iTunes\n            searches. Perhaps include misspellings of the title.",
                max_length=255,
                verbose_name="keywords",
            ),
        ),
    ]



# Generated by Django 2.1.7 on 2019-02-26 05:17

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [("cast", "0020_auto_20181121_1446")]

    operations = [
        migrations.AlterField(
            model_name="post",
            name="visible_date",
            field=models.DateTimeField(default=django.utils.timezone.now),
        )
    ]



__version__ = "0.1.19"
from pathlib import Path

default_app_config = "cast.apps.CastConfig"


def podcast_same_audio_case(audio_model, file_name, context):
    """
    podcast special case:

    Upload all audio fiels with the same name into same
    model instance, because podcast audio consists of
    different files with the same content.
    """
    stemmed_filename = Path(file_name).stem
    for audio in audio_model.objects.order_by("-created")[:10]:
        # FIXME use index instead of limit trick (maybe)
        if stemmed_filename in audio.get_audio_file_names():
            context["instance"] = audio
    return context


def upload_handler(request):
    # Apps aren't loaded yet.
    from . import models
    from filepond.forms import get_model_form

    lookup = {}
    for ending in ("jpg", "jpeg", "png", "gif"):
        lookup[ending] = (models.Image, "original", "user")

    for ending in ("wav", "webm", "ogg", "mp3", "m4a", "opus"):
        upload_field_name = ending
        if ending == "ogg":
            upload_field_name = "oga"
        lookup[ending] = (models.Audio, upload_field_name, "user")

    for ending in ("mp4", "mov", "m4v"):
        lookup[ending] = (models.Video, "original", "user")

    file_name = str(request.FILES["original"])
    ending = Path(file_name).suffix.split(".")[-1].lower()
    local_model, upload_field_name, user_field = lookup[ending]
    form_class, context = get_model_form(
        local_model, upload_field_name=upload_field_name, user_field=user_field
    )
    if local_model.__name__ == "Audio":
        context = podcast_same_audio_case(models.Audio, file_name, context)
    return form_class, context



import logging

from django.core.management.base import BaseCommand

from ...models import Post

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "add links between blogpost and media objects"

    def handle(self, *args, **options):
        posts = list(Post.objects.all().order_by("-created"))
        for post in posts:
            logger.info("-----------------")
            post.add_missing_media_objects()



# Generated by Django 2.0.9 on 2018-11-19 11:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("cast", "0009_blog_uuid")]

    operations = [
        migrations.AddField(
            model_name="blog",
            name="email",
            field=models.EmailField(
                blank=True, default=None, max_length=254, null=True
            ),
        )
    ]
